# -*- coding: utf-8 -*-

# Copyright 2013 Sebastian Ramacher <sebastian+dev@ramacher.at>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Tests for the libdiscid module
"""

# as long as Python 3.2 is supported, hack around the missing u
try:
  u = unicode
except NameError:
  u = str

class PutSuccess(object):
  first = 1
  last = 15
  sectors = 258725
  seconds = 3450
  offsets = (150, 17510, 33275, 45910, 57805, 78310, 94650, 109580, 132010,
             149160, 165115, 177710, 203325, 215555, 235590)
  track_seconds = (231, 210, 168, 159, 273, 218, 199, 299, 229, 213, 168, 342,
                   163, 267, 308)
  disc_id = u('TqvKjMu7dMliSfmVEBtrL7sBSno-')
  freedb_id = u('b60d770f')
  toc = u(' ').join(map(u, [first, last, sectors] + list(offsets)))

class _PutFail(object):
  sectors = 200
  offsets = (1, 2, 3, 4, 5, 6, 7)

class PutFail1(_PutFail):
  first = 13
  last = 1

class PutFail2(_PutFail):
  first = 0
  last = 10

class PutFail2_2(_PutFail):
  first = 100
  last = 200

class PutFail3(_PutFail):
  first = 0
  last = 0

class PutFail3_2(_PutFail):
  first = 1
  last = 100



# -*- coding: utf-8 -*-

# Copyright 2013 Sebastian Ramacher <sebastian+dev@ramacher.at>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Tests for the libdiscid module
"""

try:
  import unittest2 as unittest
except ImportError:
  import unittest
import libdiscid
import libdiscid.tests.common
from libdiscid import DiscError

# as long as Python 3.2 is supported, hack around the missing u
try:
  u = unicode
except NameError:
  u = str

class TestLibDiscId(unittest.TestCase):
  def test_version(self):
    self.assertIsNotNone(libdiscid.__version__)
    self.assertIsNotNone(libdiscid.__discid_version__)

  def test_default_device(self):
    self.assertIsNotNone(libdiscid.DEFAULT_DEVICE)

  def test_default_device_2(self):
    self.assertIsNotNone(libdiscid.default_device())

  def test_features(self):
    self.assertIsNotNone(libdiscid.FEATURES)
    self.assertIsNotNone(libdiscid.FEATURE_READ)
    self.assertIsNotNone(libdiscid.FEATURE_MCN)
    self.assertIsNotNone(libdiscid.FEATURE_ISRC)
    self.assertIsNotNone(libdiscid.FEATURES_MAPPING)

  @unittest.skipIf(libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_READ] not in
                   libdiscid.FEATURES, 'not available on this platform')
  def test_read_fail(self):
    self.assertRaises(DiscError, libdiscid.read, u('/does/not/exist'))

  def test_read_None(self):
    try:
      libdiscid.read()
    except (DiscError, NotImplementedError):
      pass

  @unittest.skipIf(libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_READ] in
                   libdiscid.FEATURES, 'available on this platform')
  def test_read_not_implemented(self):
    self.assertRaises(NotImplementedError, libdiscid.read)

  def test_put(self):
    testdata = libdiscid.tests.common.PutSuccess

    disc = libdiscid.put(testdata.first, testdata.last, testdata.sectors,
                         testdata.offsets)
    self.assertIsNotNone(disc)
    self.assertIsNone(disc.device)

    self.assertEqual(disc.id, testdata.disc_id)
    self.assertEqual(disc.freedb_id, testdata.freedb_id)
    self.assertIsNotNone(disc.submission_url)
    self.assertEqual(disc.first_track, testdata.first)
    self.assertEqual(disc.last_track, testdata.last)
    self.assertEqual(disc.sectors, testdata.sectors)
    self.assertEqual(disc.pregap, testdata.offsets[0])
    self.assertEqual(disc.leadout_track, testdata.sectors)
    self.assertEqual(disc.toc, testdata.toc)

    self.assertEqual(len(disc.track_offsets), len(testdata.offsets))
    for read_offset, expected_offset in zip(disc.track_offsets,
                                            testdata.offsets):
      self.assertEqual(read_offset, expected_offset)

    # ISRCs are not available if one calls put
    if libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_ISRC] in libdiscid.FEATURES:
      self.assertEqual(len(disc.track_isrcs), 15)
      for read_isrc in disc.track_isrcs:
        self.assertEqual(read_isrc, u(''))

    # MCN is not available if one calls put
    if libdiscid.FEATURES_MAPPING[libdiscid.FEATURE_MCN] in libdiscid.FEATURES:
      self.assertEqual(disc.mcn, u(''))

  def test_put_fail_1(self):
    # !(first < last)
    testdata = libdiscid.tests.common.PutFail1
    self.assertRaises(DiscError, libdiscid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

  def test_put_fail_2(self):
    # !(first >= 1)
    testdata = libdiscid.tests.common.PutFail2
    self.assertRaises(DiscError, libdiscid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

    # !(first < 100)
    testdata = libdiscid.tests.common.PutFail2_2
    self.assertRaises(DiscError, libdiscid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

  def test_put_fail_3(self):
    # !(last >= 1)
    testdata = libdiscid.tests.common.PutFail3
    self.assertRaises(DiscError, libdiscid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

    # !(last < 100)
    testdata = libdiscid.tests.common.PutFail3_2
    self.assertRaises(DiscError, libdiscid.put, testdata.first, testdata.last,
                      testdata.sectors, testdata.offsets)

  def test_sectors_to_seconds(self):
    # there are 75 sectors per second
    for sectors in range(38):
      # round down for sectors in the first half
      self.assertEqual(libdiscid.sectors_to_seconds(sectors), 0)
    for sectors in range(38, 76):
      # round up for sectors in the second half
      self.assertEqual(libdiscid.sectors_to_seconds(sectors), 1)

if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-

# Copyright 2013 Sebastian Ramacher <sebastian+dev@ramacher.at>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Python bindings for libdiscid

libdiscid is a library to calculate MusicBrainz Disc IDs.
This module provides Python-bindings for libdiscid.

>>> disc = libdiscid.read()
>>> disc.id is not None
True
"""

from __future__ import division

import libdiscid._discid
from libdiscid.exceptions import DiscError
import re
import warnings

__version__ = '0.4.1'

DEFAULT_DEVICE = libdiscid._discid.default_device()
""" The default device to use for :func:`DiscId.read` on this platform.

.. deprecated:: 0.2.0
   Please use :func:`default_device` instead.
"""

FEATURES = libdiscid._discid.FEATURES
""" List of all available features supported by libdiscid on this platform.
"""
FEATURE_READ = libdiscid._discid.FEATURE_READ
""" Read the TOC of the disc to get the disc ID. This feature is always enabled.
"""
FEATURE_MCN = libdiscid._discid.FEATURE_MCN
"""  Read the Media Catalogue Number of the disc.
"""
FEATURE_ISRC = libdiscid._discid.FEATURE_ISRC
""" Read :musicbrainz:`International Standard Recording Codes <ISRC>` of all the
    tracks.
"""
FEATURES_MAPPING = libdiscid._discid.FEATURES_MAPPING
""" Mapping between the constants representing a feature and their string
    representation.
"""
__discid_version__ = libdiscid._discid.__discid_version__
""" The version of the underlying libdiscid.
"""

class DiscId(object):
  """ Disc information

  Class holding all the information obtained from a disc.
  """

  def __init__(self, cdiscid):
      self._id = cdiscid.id
      self._freedb_id = cdiscid.freedb_id
      self._submission_url = cdiscid.submission_url
      self._webservice_url = cdiscid.webservice_url
      self._first_track = cdiscid.first_track
      self._last_track = cdiscid.last_track
      self._sectors = cdiscid.sectors
      self._track_offsets = cdiscid.track_offsets
      self._track_lengths = cdiscid.track_lengths
      self._mcn = cdiscid.mcn
      self._track_isrcs = cdiscid.track_isrcs
      self._device = cdiscid.device
      self._toc = cdiscid.toc

  @property
  def id(self):
    """ The MusicBrainz :musicbrainz:`Disc ID`.
    """

    return self._id

  @property
  def freedb_id(self):
    """ The :musicbrainz:`FreeDB` Disc ID (without category).
    """

    return self._freedb_id

  @property
  def submission_url(self):
    """ Disc ID / TOC Submission URL for MusicBrainz

    With this url you can submit the current TOC as a new MusicBrainz
    :musicbrainz:`Disc ID`.
    """

    return self._submission_url

  @property
  def webservice_url(self):
    """ The web service URL for info about the CD

    With this url you can retrive information about the CD in XML from the
    MusicBrainz web service.
    """

    warnings.warn('webservice_url is deprecated since it points to the old '
                  'webservice. Please use python-musicbrainz-ngs to access '
                  'the webservice.', DeprecationWarning)
    return self._webservice_url

  @property
  def first_track(self):
    """ Number of the first audio track.
    """

    return self._first_track

  @property
  def last_track(self):
    """ Number of the last audio track.
    """

    return self._last_track

  @property
  def sectors(self):
    """ Total sector count.
    """

    return self._sectors

  @property
  def leadout_track(self):
    """ Leadout track.
    """

    return self.sectors

  @property
  def track_offsets(self):
    """ Tuple of all track offsets (in sectors).

    The first element corresponds to the offset of the track denoted by
    :attr:`first_track` and so on.
    """

    return self._track_offsets

  @property
  def pregap(self):
    """ Pregap of the first track (in sectors).
    """

    return self.track_offsets[0]


  @property
  def track_lengths(self):
    """ Tuple of all track lengths (in sectors).

    The first element corresponds to the length of the track denoted by
    :attr:`first_track` and so on.
    """

    return self._track_lengths

  @property
  def mcn(self):
    """ Media Catalogue Number of the disc.

    :raises NotImplementedError: reading MCN is not supported on this platform
    """

    if self._mcn is None:
      raise NotImplementedError('MCN is not available with this version '
                                'of libdiscid and/or platform')
    return self._mcn

  @property
  def track_isrcs(self):
    """ Tuple of :musicbrainz:`ISRCs <ISRC>` of all tracks.

    The first element of the list corresponds to the ISRC of the
    :attr:`first_track` and so on.

    :raises NotImplementedError: reading ISRCs is not supported on this platform
    """

    if self._track_isrcs is None:
      raise NotImplementedError('ISRC is not available with this version '
                                'of libdiscid and/or platform')
    return self._track_isrcs

  @property
  def device(self):
    """ The device the data was read from.

    If it is ``None``, :func:`libdiscid.put` was called to create the instance.
    """

    return self._device

  @property
  def toc(self):
    """ String representing the CD's Table of Contents (TOC).

    :raises ValueError: extracting TOC string from the submission URL failed
    """

    if self._toc is None:
      # extract TOC string from submission URL
      match = re.match(r'.*toc=([0-9+]+)$', self.submission_url)
      if match is None:
        raise ValueError('Failed to extract TOC from submission URL')
      self._toc = match.group(1).replace('+', ' ')
    return self._toc

def read(device=None, features=None):
  """ Reads the TOC from the device given as string.

  If *device* is ``None``, :func:`default_device` is used to determine
  the device. *features* can be any combination of :data:`FEATURE_MCN` and
  :data:`FEATURE_ISRC` and :data:`FEATURE_READ`. Note that prior to libdiscid
  version 0.5.0 *features* has no effect and that :data:`FEATURE_READ` is always
  assumed, even if not given.

  :param device: device to read from
  :type device: unicode or None
  :param features: selected features, possible values are :data:`FEATURE_READ` \
    :data:`FEATURE_MCN`, :data:`FEATURE_ISRC` and any of these values combined \
    with bitwise or.
  :type features: integer or None
  :raises libdiscid.DiscError: reading the disc failed
  :raises NotImplementedError: reading discs is not supported
  :raises MemoryError: failed to allocate the internal DiscId object
  :rtype: :class:`DiscId` object
  """

  disc = libdiscid._discid.DiscId()
  if features is None:
    disc.read(device)
  else:
    disc.read(device, features)
  return DiscId(disc)

def put(first, last, sectors, offsets):
  """ Creates a TOC based on the given offets.

  Takes the *first* and *last* audio track, as well as the number of
  *sectors* and a list of *offsets* as in :attr:`track_offsets`.

  :param first: number of the first audio track
  :type first: integer
  :param last: number of the last audio track
  :type last: integer
  :param sectors: total number of sectors on the disc
  :type sectors: integer
  :param offsets: offsets of each track
  :type offsets: list or tuple of integers
  :raises libdiscid.DiscError: operation failed for some reason
  :raises MemoryError: failed to allocated memory to store the offsets or the \
    internal DiscId object
  :rtype: :class:`DiscId` object
  """

  disc = libdiscid._discid.DiscId()
  disc.put(first, last, sectors, offsets)
  return DiscId(disc)

def default_device():
  """ The default device on this platform.

  The default device can change during the run-time of the program. This can
  happen with removable devices for example.

  :rtype: unicode
  """

  return libdiscid._discid.default_device()

def sectors_to_seconds(sectors):
  """ Convert sectors to seconds rounded to the nearest second.

  :param sectors: number of sectors
  :type sectors: integer
  :rtype: integer
  """

  SECTORS_PER_SECOND = 75
  remainder = sectors % SECTORS_PER_SECOND
  return sectors // SECTORS_PER_SECOND + \
    (1 if remainder > SECTORS_PER_SECOND // 2 else 0)

__all__ = (
  'read', 'put', 'default_device', 'sectors_to_seconds',
  '__version__', '__discid_version__',
  'FEATURES', 'FEATURES_MAPPING', 'FEATURE_READ', 'FEATURE_MCN', 'FEATURE_ISRC',
  'DEFAULT_DEVICE',
  'DiscId', 'DiscError'
)



# -*- coding: utf-8 -*-

# Copyright 2013 Sebastian Ramacher <sebastian+dev@ramacher.at>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the “Software”), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

cimport cdiscid
cimport cpython
from libc cimport limits
from libc.stdlib cimport malloc, free
from cpython cimport bool
from libdiscid.exceptions import DiscError

cdef bool _has_feature(int feature):
  return cdiscid.wrap_has_feature(feature) == 1

cdef unicode _to_unicode(char* s):
  return s.decode('UTF-8', 'strict')

cdef class DiscId:
  """ Class to calculate MusicBrainz Disc IDs.

  >>> d = DiscId()
  >>> d.read()
  >>> d.id is not None
  True

  Note that all the properties are only set after an successful call to
  :func:`DiscId.read` or :func:`DiscId.put`.
  """

  cdef cdiscid.DiscId *_c_discid
  cdef bool _have_read
  cdef unicode _device

  def __cinit__(self):
    self._c_discid = cdiscid.discid_new()
    if self._c_discid is NULL:
      raise MemoryError('Failed to allocate DiscId object')

    self._have_read = False
    self._device = None

  def __dealloc__(self):
    if self._c_discid is not NULL:
      cdiscid.discid_free(self._c_discid)

  cdef _read(self, char* device, unsigned int features):
    if not _has_feature(cdiscid.DISCID_FEATURE_READ):
      raise NotImplementedError('read is not available with this version of '
                                'libdiscid and/or platform')

    if not cdiscid.wrap_read_sparse(self._c_discid, device, features):
      raise DiscError(self._get_error_msg())
    self._have_read = True

  def read(self, unicode device=None, unsigned int features=limits.UINT_MAX):
    """ Reads the TOC from the device given as string.

    If *device* is ``None``, :func:`libdiscid.default_device` is used.
    *features* can be any combination of :data:`FEATURE_MCN` and
    :data:`FEATURE_ISRC` and :data:`FEATURE_READ`. Note that prior to libdiscid
    version 0.5.0 *features* has no effect and that :data:`FEATURE_READ` is
    always assumed, even if not given.

    A :exc:`libdiscid.DiscError` exception is raised when reading fails, and
    :py:exc:`NotImplementedError` when libdiscid does not support reading discs
    on the current platform.
    """

    if device is None:
      device = default_device()

    py_byte_device = device.encode('UTF-8')
    cdef char* cdevice = py_byte_device
    ret = self._read(cdevice, features)
    self._device = device

  cdef _put(self, int first, int last, int* offsets):
    if not cdiscid.discid_put(self._c_discid, first, last, offsets):
      raise DiscError(self._get_error_msg())
    self._device = None
    self._have_read = True

  def put(self, int first, int last, int sectors, offsets):
    """ Creates a TOC based on the given offets.

    Takes the *first* and *last* audio track, as well as the number of
    *sectors* and a list of *offsets* as in :attr:`DiscId.track_offsets`.

    If the operation fails for some reason, a :exc:`libdiscid.DiscError`
    exception is raised.
    """

    cdef int* coffsets = <int*> malloc((len(offsets) + 1) * sizeof(int))
    if coffsets is NULL:
      raise MemoryError('Failed to allocate memory to store offsets')

    try:
      coffsets[0] = sectors
      for (i, v) in enumerate(offsets):
        coffsets[i + 1] = v
      return self._put(first, last, coffsets)
    finally:
      free(coffsets)

  cdef unicode _get_error_msg(self):
    return _to_unicode(cdiscid.discid_get_error_msg(self._c_discid))

  property id:
    """ The MusicBrainz :musicbrainz:`Disc ID`.
    """

    def __get__(self):
      return _to_unicode(cdiscid.discid_get_id(self._c_discid))

  property freedb_id:
    """ The :musicbrainz:`FreeDB` Disc ID (without category).
    """

    def __get__(self):
      return _to_unicode(cdiscid.discid_get_freedb_id(self._c_discid))

  property submission_url:
    """ Disc ID / TOC Submission URL for MusicBrainz

    With this url you can submit the current TOC as a new MusicBrainz
    :musicbrainz:`Disc ID`.
    """

    def __get__(self):
      return _to_unicode(cdiscid.discid_get_submission_url(self._c_discid))

  property webservice_url:
    """ The web service URL for info about the CD

    With this url you can retrive information about the CD in XML from the
    MusicBrainz web service.
    """

    def __get__(self):
      return _to_unicode(cdiscid.discid_get_webservice_url(self._c_discid))

  property first_track:
    """ Number of the first audio track.
    """

    def __get__(self):
      return cdiscid.discid_get_first_track_num(self._c_discid)

  property last_track:
    """ Number of the last audio track.
    """

    def __get__(self):
      return cdiscid.discid_get_last_track_num(self._c_discid)

  property sectors:
    """ Total sector count.
    """

    def __get__(self):
      return cdiscid.discid_get_sectors(self._c_discid)

  property track_offsets:
    """ Tuple of all track offsets (in sectors).

    The first element corresponds to the offset of the track denoted by
    :attr:`first_track` and so on.
    """

    def __get__(self):
      return tuple(cdiscid.discid_get_track_offset(self._c_discid, track)
                   for track in range(self.first_track, self.last_track + 1))

  property track_lengths:
    """ Tuple of all track lengths (in sectors).

    The first element corresponds to the length of the track denoted by
    :attr:`first_track` and so on.
    """

    def __get__(self):
      return tuple(cdiscid.discid_get_track_length(self._c_discid, track)
                   for track in range(self.first_track, self.last_track + 1))

  property mcn:
    """ Media Catalogue Number of the disc.
    """

    def __get__(self):
      if not _has_feature(cdiscid.DISCID_FEATURE_MCN):
        return None
      return _to_unicode(cdiscid.wrap_get_mcn(self._c_discid))

  property track_isrcs:
    """ Tuple of :musicbrainz:`ISRCs <ISRC>` of all tracks.

    The first element of the list corresponds to the ISRC of the
    :attr:`first_track` and so on.
    """

    def __get__(self):
      if not _has_feature(cdiscid.DISCID_FEATURE_ISRC):
        return None
      return tuple(_to_unicode(cdiscid.wrap_get_track_isrc(self._c_discid,
                                                           track)) for
                   track in range(self.first_track, self.last_track + 1))

  property device:
    """ The device the data was read from.

    If it is ``None``, :func:`libdiscid.put` was called to create the instance.
    """

    def __get__(self):
      return self._device

  property toc:
    """ String representing the CD's Table of Contents (TOC).
    """

    def __get__(self):
      assert self._have_read

      cdef char* tocstr = cdiscid.wrap_get_toc(self._c_discid)
      if tocstr is not NULL:
        return _to_unicode(tocstr)
      return None

FEATURES_MAPPING = {
    cdiscid.DISCID_FEATURE_READ: _to_unicode(cdiscid.DISCID_FEATURE_STR_READ),
    cdiscid.DISCID_FEATURE_MCN: _to_unicode(cdiscid.DISCID_FEATURE_STR_MCN),
    cdiscid.DISCID_FEATURE_ISRC: _to_unicode(cdiscid.DISCID_FEATURE_STR_ISRC)
}

cdef _feature_list():
  res = []
  for f, s in FEATURES_MAPPING.items():
    if _has_feature(f):
      res.append(s)
  return res

def default_device():
  """ The default device on this platform.
  """

  return _to_unicode(cdiscid.discid_get_default_device())

FEATURES = _feature_list()
FEATURE_READ = cdiscid.DISCID_FEATURE_READ
FEATURE_MCN = cdiscid.DISCID_FEATURE_MCN
FEATURE_ISRC = cdiscid.DISCID_FEATURE_ISRC
__discid_version__ = _to_unicode(cdiscid.wrap_get_version_string())




#!/usr/bin/python

import os.path
import sys

try:
  from setuptools import setup, Extension
  have_setuptools = True
except ImportError:
  from distutils.core import setup
  from distutils.extension import Extension
  have_setuptools = False

try:
  from Cython.Build import cythonize
  have_cython = True
except ImportError:
  have_cython = False

if have_cython:
  # if Cython is available, rebuild _discid.c
  ext = cythonize([
    Extension('libdiscid._discid',
      [
        'libdiscid/_discid.pyx',
        'libdiscid/discid-wrapper.c'
      ]
    )
  ])
else:
  # ... otherwise use the shipped version of _discid.c
  ext = [
    Extension('libdiscid._discid',
      [
        'libdiscid/_discid.c',
        'libdiscid/discid-wrapper.c'
      ]
    )
  ]

if have_setuptools:
  tests_require = []
  if sys.version_info[0:2] < (2,7):
    tests_require = ['unittest2']

  args = {
    'test_suite': 'libdiscid.tests',
    'tests_require': tests_require,
  }

  if have_cython:
    # if Cython is available, check if it's new enough
    args['setup_requires'] = ['cython >= 0.15']
else:
  args = {}

def read(name):
  f = open(os.path.join(os.path.dirname(__file__), name))
  ret = f.read()
  f.close()
  return ret

setup(
  name='python-libdiscid',
  version='0.4.1',
  description='Python bindings for libdiscid',
  long_description=read('README.rst'),
  author='Sebastian Ramacher',
  author_email='sebastian+dev@ramacher.at',
  url='https://github.com/sebastinas/python-libdiscid',
  license='Expat',
  ext_modules=ext,
  packages=[
    'libdiscid',
    'libdiscid.tests',
    'libdiscid.compat'
  ],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Topic :: Multimedia :: Sound/Audio :: CD Audio :: CD Ripping',
    'Topic :: Software Development :: Libraries :: Python Modules'
  ],
  **args
)
